﻿using System;
using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using MeshVR;
using Request = MeshVR.AssetLoader.AssetBundleFromFileRequest;
using AssetBundles;

// LoadTrigger Plugin by Captain Varghoss
// Very simple plugin to detect when a scene is fully loaded then initiates a trigger of your choosing.
// Configurable delay between scene loaded and trigger.
// Huge thanks to hazmhox, MacGruber and AcidBubbles for their work with custom triggers!


namespace CaptainVarghoss {
	public class LoadTrigger : MVRScript {
		protected JSONStorableFloat howLongDoWeWait;
		protected JSONStorableStringChooser whatDoWeTrigger = null;
		private EventTrigger loadTriggerTrigger;
		private List<EventTrigger> allTriggers;
		private static bool areWeThereYet = false;
		private static int howMuchTimeIsLeft = 30;
		private static float startTime = 0;
		
		public override void Init() {
			try {
				// Float slider for delay time
				howLongDoWeWait = new JSONStorableFloat("Number of Seconds to Delay", 30f, 1f, 100f);
				RegisterFloat(howLongDoWeWait);
				CreateSlider(howLongDoWeWait, false);
				
				// Make a button trigger here 
				UIDynamicButton editTriggerBtn = CreateButton("Edit Trigger", false);
				if (editTriggerBtn != null) {
					editTriggerBtn.button.onClick.AddListener( () => { loadTriggerTrigger.OpenPanelActionStart(); } );
				}
				
				allTriggers = new List<EventTrigger>();
				loadTriggerTrigger = new EventTrigger(this, "LoadTriggerTrigger");
				allTriggers.Add(loadTriggerTrigger);
				SimpleTriggerHandler.LoadAssets();
				
				areWeThereYet = false;
				startTime = 0;
//				SuperController.LogMessage("LoadTrigger - Loaded");
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}
		
		private void OnAtomRename(string oldid, string newid)
		{
			foreach( EventTrigger et in allTriggers ) {
				et.SyncAtomNames();
			}
		}
		
		void Start() {
			try {
				// Set our start time
				startTime = Time.time;
				areWeThereYet = false;
//				SuperController.LogMessage(startTime.ToString());
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		void Update() {
			try {
				if (areWeThereYet == false)
				{
					if (Time.time - startTime > howLongDoWeWait.val)
					{
//						SuperController.LogMessage("LoadTrigger - Done Waiting ("+howLongDoWeWait.val+" Seconds)");
						if (SuperController.singleton.isLoading == false)
						{
							loadTriggerTrigger.Trigger();
//							SuperController.LogMessage("Scene Loaded");
							areWeThereYet = true;
						}
					}
				}
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		void FixedUpdate() {
			try {
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}


		// ===========================================================================================
		// TriggerHandler implementation for easier handling of custom triggers.
		// Essentially call this in your plugin init code:
		//     StartCoroutine(SimpleTriggerHandler.LoadAssets());
		//
		// Credit to AcidBubbles for figuring out how to do custom triggers.
		//
		// hazmhox note : Edited a bit to allow the use of transitions ( original code coming from LogicBricks 4 )
		// Captain Varghoss note: blatantly stolen from VAMMoan :P
		public class SimpleTriggerHandler : TriggerHandler
		{
			public static bool Loaded { get; private set; }

			private static SimpleTriggerHandler myInstance;

			private RectTransform myTriggerActionsPrefab;
			private RectTransform myTriggerActionMiniPrefab;
			private RectTransform myTriggerActionDiscretePrefab;
			private RectTransform myTriggerActionTransitionPrefab;

			public static SimpleTriggerHandler Instance {
				get {
					if (myInstance == null)
						myInstance = new SimpleTriggerHandler();
					return myInstance;
				}
			}

			public static void LoadAssets()
			{
				SuperController.singleton.StartCoroutine(Instance.LoadAssetsInternal());
			}

			private IEnumerator LoadAssetsInternal()
			{
				foreach (var x in LoadAsset("z_ui2", "TriggerActionsPanel", p => myTriggerActionsPrefab = p))
					yield return x;
				foreach (var x in LoadAsset("z_ui2", "TriggerActionMiniPanel", p => myTriggerActionMiniPrefab = p))
					yield return x;
				foreach (var x in LoadAsset("z_ui2", "TriggerActionDiscretePanel", p => myTriggerActionDiscretePrefab = p))
					yield return x;
				foreach (var x in LoadAsset("z_ui2", "TriggerActionTransitionPanel", p => myTriggerActionTransitionPrefab = p))
					yield return x;

				Loaded = true;
			}

			private IEnumerable LoadAsset(string assetBundleName, string assetName, Action<RectTransform> assign)
			{
				AssetBundleLoadAssetOperation request = AssetBundleManager.LoadAssetAsync(assetBundleName, assetName, typeof(GameObject));
				if (request == null)
					throw new NullReferenceException($"Request for {assetName} in {assetBundleName} assetbundle failed: Null request.");
				yield return request;
				GameObject go = request.GetAsset<GameObject>();
				if (go == null)
					throw new NullReferenceException($"Request for {assetName} in {assetBundleName} assetbundle failed: Null GameObject.");
				RectTransform prefab = go.GetComponent<RectTransform>();
				if (prefab == null)
					throw new NullReferenceException($"Request for {assetName} in {assetBundleName} assetbundle failed: Null RectTansform.");
				assign(prefab);
			}


			void TriggerHandler.RemoveTrigger(Trigger t)
			{
				// nothing to do
			}

			void TriggerHandler.DuplicateTrigger(Trigger t)
			{
				throw new NotImplementedException();
			}

			RectTransform TriggerHandler.CreateTriggerActionsUI()
			{
				return UnityEngine.Object.Instantiate(myTriggerActionsPrefab);
			}

			RectTransform TriggerHandler.CreateTriggerActionMiniUI()
			{
				return UnityEngine.Object.Instantiate(myTriggerActionMiniPrefab);
			}

			RectTransform TriggerHandler.CreateTriggerActionDiscreteUI()
			{
				return UnityEngine.Object.Instantiate(myTriggerActionDiscretePrefab);
			}

			RectTransform TriggerHandler.CreateTriggerActionTransitionUI()
			{
				return UnityEngine.Object.Instantiate(myTriggerActionTransitionPrefab);
			}

			void TriggerHandler.RemoveTriggerActionUI(RectTransform rt)
			{
				UnityEngine.Object.Destroy(rt?.gameObject);
			}
		}

		// Wrapper for easier handling of custom triggers.
		public class EventTrigger : Trigger
		{
			public string Name {
				get { return name; }
				set { name = value; myNeedInit = true; }
			}

			public MVRScript Owner {
				get; private set;
			}

			private string name;
			private bool myNeedInit = true;

			public EventTrigger(MVRScript owner, string name)
			{
				Name = name;
				Owner = owner;
				handler = SimpleTriggerHandler.Instance;
			}

			public void OpenPanelActionStart()
			{
				if (!SimpleTriggerHandler.Loaded)
				{
					SuperController.LogError("EventTrigger: You need to call SimpleTriggerHandler.LoadAssets() before use.");
					return;
				}

				triggerActionsParent = Owner.UITransform;
				InitTriggerUI();
				OpenTriggerActionsPanel();
				if (myNeedInit)
				{
					triggerActionsPanel.Find("Panel/Header Text").GetComponent<Text>().text = Name;
					triggerActionsPanel.Find("Content/Tab1/Label").GetComponent<Text>().text = "Event Actions";
					triggerActionsPanel.Find("Content/Tab2").gameObject.SetActive(false);
					triggerActionsPanel.Find("Content/Tab3").gameObject.SetActive(false);
					myNeedInit = false;
				}
			}
			
			public void OpenPanelTransition()
			{
				if (!SimpleTriggerHandler.Loaded)
				{
					SuperController.LogError("EventTrigger: You need to call SimpleTriggerHandler.LoadAssets() before use.");
					return;
				}

				triggerActionsParent = Owner.UITransform;
				InitTriggerUI();
				OpenTriggerActionsPanel();
				
				//getRoot(triggerActionsParent,0);
				if (myNeedInit)
				{
					triggerActionsPanel.Find("Panel/Header Text").GetComponent<Text>().text = Name;
					triggerActionsPanel.Find("Content/Tab2/Label").GetComponent<Text>().text = "Breathing Actions";
					triggerActionsPanel.Find("Content/Tab1").gameObject.SetActive(false);
					triggerActionsPanel.Find("Content/Tab2").gameObject.SetActive(true);
					triggerActionsPanel.Find("Content/Tab2").GetComponent<Toggle>().isOn = true;					
					triggerActionsPanel.Find("Content/Tab3").gameObject.SetActive(false);
					myNeedInit = false;
				}
			}
	
			public void RestoreFromJSON(JSONClass jc, string subScenePrefix, bool isMerge, bool setMissingToDefault)
			{
				if (jc.HasKey(Name))
				{
					JSONClass tc = jc[Name].AsObject;
					if (tc != null)
						base.RestoreFromJSON(tc, subScenePrefix, isMerge);
				}
				else if (setMissingToDefault)
				{
					base.RestoreFromJSON(new JSONClass());
				}
			}

			public void Trigger()
			{
				active = true;
				active = false;
			}
			
			public void TriggerTransition(float val)
			{
				this.transitionInterpValue = val;
			}
		}
		
		// More code stolen from VAMMoan
		// **************************
		// OVERRIDES FOR THE TRIGGERS
		// **************************	
		public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
		{
			JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
			if (includePhysical || forceStore)
			{
				needsStore = true;
				foreach( EventTrigger et in allTriggers ) {
					jc[et.Name] = et.GetJSON(base.subScenePrefix);
				}
				
			}
			return jc;
		}

		public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
		{
			base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);
			if (!base.physicalLocked && restorePhysical && !IsCustomPhysicalParamLocked("trigger"))
			{
				foreach( EventTrigger et in allTriggers ) {
					et.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
				}
			}
		}		
		// OnDestroy is where you should put any cleanup
		// if you registered objects to supercontroller or atom, you should unregister them here
		void OnDestroy() {
		}

	}
}